from bs4 import BeautifulSoup
from urllib.parse import urlparse
import os
import re

def shortcodify(body: str):
    """Replace post-body HTML with Hugo shortcodes when possible."""
    soup = BeautifulSoup(body, 'html.parser')
    images_src = []
    
    # <figure> -> {{% figure %}}
    for fig in soup('figure'):
        link = fig.find('a')
        href = None
        caption = None
        src = None
        if link:
            # This image is wrapped in a link
            href = link.get('href')
            link.unwrap() # remove link but keep contents
        img = fig.find('img')
        if img:
            src = img.get('src')
            images_src.append(src) # add to our list
            if os.path.basename(urlparse(src).path):
                # the IMG SRC is a file, so let's just use the filename sans path
                src = os.path.basename(urlparse(src).path)
                src = re.sub(r'(.*)\-(\d+x\d+)\.(jpg|png)$', r'\1.\3', src)
        figcaption = fig.find('figcaption')
        if figcaption:
            caption = figcaption.text

        shortcode = '{{% figure'
        if src:
            shortcode += ' src="{}"'.format(src)
        if href:
            shortcode += ' link="{}"'.format(href)
        if caption:
            shortcode += ' caption="{}"'.format(caption)

        shortcode += ' %}}'
        fig.insert_before(shortcode)
        fig.decompose() # destroy original figure tag

    # <img> -> {{% img %}}
    # Also maps Wordpress class="alignright" to class="float-right", etc. 
    for img in soup('img'):
        link = img.findParent('a')
        href = None
        if link:
            # This image is wrapped in a link
            href = link.get('href')
            link.unwrap() # remove link but keep contents
        src = img.get('src')
        images_src.append(src) # add to our list
        alt = img.get('alt')
        if os.path.basename(urlparse(src).path):
            # the IMG SRC is a file, so let's just use the filename sans path
            src = os.path.basename(urlparse(src).path)
            src = re.sub(r'(.*)\-(\d+x\d+)\.(jpg|png)$', r'\1.\3', src)
        shortcode = '{{% img'
        shortcode += ' src="{}"'.format(src) if src else ''
        if alt:
            shortcode += ' alt="{}"'.format(alt)
        if href:
            shortcode += ' link="{}"'.format(href)
        if 'alignright' in img.get('class', []):
            shortcode += ' class="float-right"'
        if 'alignleft' in img.get('class', []):
            shortcode += ' class="float-left"'
        shortcode += ' %}}'
        img.insert_before(shortcode)
        img.decompose() # destroy original img tag

    # <blockquote> -> {{% blockquote %}}
    for bq in soup('blockquote'):
        cite = bq.cite
        if cite:
            citation = cite.extract().decode_contents()
        shortcode = '{{% blockquote'
        if cite:
            shortcode += ' source="{}"'.format(citation.replace('"', '&quot;'))
        shortcode += ' %}}\n'
        bq.insert_before(shortcode)
        bq.insert_after('\n{{% /blockquote %}}')
        bq.unwrap() # destroy original blockquote tag

    return str(soup), images_src


import re
import datetime
from urllib.parse import urlparse
from xml.etree import ElementTree
import pytz
import tzlocal

def parse_pubdate(datestr):
    # Draft posts often have a date year of -0001. Awesome.
    datestr = re.sub(r'\+000$', '+0000', datestr)
    datestr = re.sub(r'-0001', '1970', datestr)
    return datetime.datetime.strptime(datestr, '%a, %d %b %Y %H:%M:%S %z')


class WXRFile(object):
    def __init__(self, input_file, blog_timezone=None):
        self.input_file = input_file
        # Parse once to fetch XML namespaces
        self.namespaces = dict([
            node for _, node in ElementTree.iterparse(
                input_file, events=['start-ns'])
        ])
        # Parse again to build tree
        self.tree = ElementTree.parse(input_file)
        self.blog_title = self.tree.find('channel').findtext('title')
        self.blog_url = self.tree.find('channel').findtext('link')
        self.blog_hostname = urlparse(
            self.tree.find('channel').findtext('link')).hostname
        self.blog_language = self.tree.find('channel').findtext('language')

        # Allow users to override timezone
        if blog_timezone:
            self.blog_timezone = pytz.timezone(blog_timezone)
        else:
            self.blog_timezone = tzlocal.get_localzone()

    def get_attachments(self):
        return [
            i for i in self.get_items() if i.get('post_type') == 'attachment'
        ]

    def get_nav_menu_items(self):
        return [
            i for i in self.get_items()
            if i.get('post_type') == 'nav_menu_item'
        ]

    def get_pages(self):
        return [i for i in self.get_items() if i.get('post_type') == 'page']

    def get_posts(self):
        return [i for i in self.get_items() if i.get('post_type') == 'post']

    def get_items(self):
        for post in self.tree.find('channel').findall('item'):
            out = {}

            inputs = [
                ('title', 'title'),
                ('link', 'link'),
                ('pubDate', 'pubDate'),
                ('content:encoded', 'body'),
                ('excerpt:encoded', 'excerpt'),
                ('wp:post_id', 'post_id'),
                ('wp:post_date', 'post_date'),
                ('wp:post_date_gmt', 'post_date_gmt'),
                ('wp:post_name', 'post_name'),
                ('wp:status', 'status'),
                ('wp:comment_status', 'comment_status'),
                ('wp:post_type', 'post_type'),
                ('wp:post_parent', 'post_parent'),
                ('wp:attachment_url', 'attachment_url'),
                ('dc:creator', 'author'),
            ]
            for a, b in inputs:
                if post.findtext(a, namespaces=self.namespaces):
                    out[b] = post.findtext(
                        a, namespaces=self.namespaces).strip()

            if 'pubDate' in out:
                out['pubDate'] = parse_pubdate(out['pubDate'])
            if 'post_date' in out:
                out['post_date'] = self.blog_timezone.localize(datetime.datetime.strptime(out.get('post_date')[:19], '%Y-%m-%d %H:%M:%S'))
            if 'post_date_gmt' in out:
                if out['post_date_gmt'].startswith('0000'):
                    out['post_date_gmt'] = datetime.datetime(1970, 1, 1, 0, 0, 0, tzinfo=datetime.timezone.utc)
                else:
                    out['post_date_gmt'] = datetime.datetime.strptime(out.get('post_date_gmt')[:19], '%Y-%m-%d %H:%M:%S').replace(tzinfo=datetime.timezone.utc)
            if 'post_id' in out:
                out['post_id'] = int(out['post_id'])
            if 'post_parent' in out:
                out['post_parent'] = int(out['post_parent'])

            postmeta = {}
            for m in post.findall('wp:postmeta', namespaces=self.namespaces):
                k = m.findtext('wp:meta_key', namespaces=self.namespaces)
                v = m.findtext('wp:meta_value', namespaces=self.namespaces)
                postmeta[k] = v

            if postmeta:
                out['postmeta'] = postmeta

            categories = []
            tags = []
            for c in post.findall('category'):
                if c.get('domain') == 'category':
                    categories.append(c.text)
                elif c.get('domain') == 'post_tag':
                    tags.append(c.text)
                elif c.get('domain') == 'post_format':
                    # for some reason post_format is encoded as a category
                    out['post_format'] = c.get('nicename').replace('post-format-', '')

            if categories:
                out['categories'] = categories
            if tags:
                out['tags'] = tags

            comments = []
            for c in post.findall('wp:comment', namespaces=self.namespaces):
                comment = {}
                comment_inputs = [('wp:comment_id', 'comment_id'),
                                  ('wp:comment_author', 'comment_author'),
                                  ('wp:comment_author_email',
                                   'comment_author_email'),
                                  ('wp:comment_author_url',
                                   'comment_author_url'),
                                  ('wp:comment_author_IP',
                                   'comment_author_IP'),
                                  ('wp:comment_date', 'comment_date'),
                                  ('wp:comment_date_gmt', 'comment_date_gmt'),
                                  ('wp:comment_content', 'comment_content'),
                                  ('wp:comment_approved', 'comment_approved'),
                                  ('wp:comment_type', 'comment_type'),
                                  ('wp:comment_parent', 'comment_parent'),
                                  ('wp:comment_user_id', 'comment_user_id')]
                for a, b in comment_inputs:
                    if c.findtext(a, namespaces=self.namespaces):
                        comment[b] = c.findtext(
                            a, namespaces=self.namespaces).strip()

                if 'comment_date' in comment:
                    comment['comment_date'] = datetime.datetime.strptime(
                        comment['comment_date'], '%Y-%m-%d %H:%M:%S')
                if 'comment_date_gmt' in comment:
                    comment['comment_date_gmt'] = datetime.datetime.strptime(
                        comment['comment_date_gmt'],
                        '%Y-%m-%d %H:%M:%S').replace(
                            tzinfo=datetime.timezone.utc)
                if 'comment_approved' in comment:
                    comment['comment_approved'] = bool(
                        comment['comment_parent'])
                if 'comment_id' in comment:
                    comment['comment_id'] = int(comment['comment_id'])
                if 'comment_parent' in comment:
                    comment['comment_parent'] = int(comment['comment_parent'])
                if 'comment_user_id' in comment:
                    comment['comment_user_id'] = int(
                        comment['comment_user_id'])
                comments.append(comment)

            if comments:
                out['comments'] = comments

            yield out



#!/usr/bin/env python3

import click
import time
from .wxrfile import WXRFile
from .autop import wpautop
from .wp_shortcodes import parse_shortcodes
import html2text
from .hugo_shortcodes import shortcodify
from urllib.parse import urlparse, urljoin
import json
import requests
import dateutil.parser

import os
import datetime
import hashlib
import re
import sys
from ruamel.yaml import YAML  # import yaml

yaml = YAML()
yaml.default_flow_style = False


def contains_wp_shortcodes(body, *sc):
    """Search body for one or more shortcodes named sc"""
    tagre = '|'.join(sc)
    # terrible regex taken from Wordpress source
    pattern = re.compile(
        '\\[(\\[?)(' + tagre +
        ')\\b([^\\]\\/]*(?:\\/(?!\\])[^\\]\\/]*)*?)(?:(\\/)\\]|\\](?:([^\\[]*(?:\\[(?!\\/\\2\\])[^\\[]*)*)\\[\\/\\2\\])?)(\\]?)'
    )
    if body:
        return pattern.findall(body)
    else:
        return False


def check_post_attachments(post: dict, allattach: dict):
    # Scan HTML body for <img> tags, presuming we'll download these
    if re.search(r'<img\s', post.get('body', '')):
        post['hugo_has_attachments'] = True
    # Also check for attachments known to Wordpress
    if [p for p in allattach if p.get('post_parent') == post.get('post_id')]:
        post['hugo_has_attachments'] = True
    return


def make_post_destinations(post: dict):
    post_date = post.get('post_date', datetime.datetime(1970, 1, 1, 0, 0, 0))
    fn = '{}-{}'.format(
        post_date.strftime('%Y-%m-%d'), post.get('post_name', 'UNTITLED'))
    if post.get('hugo_has_attachments'):
        filepath = os.path.join('posts', fn, 'index.md')
        bundlepath = os.path.join('posts', fn)
    else:
        filepath = os.path.join('posts', fn + '.md')
        bundlepath = None
    post['hugo_filepath'] = filepath
    post['hugo_bundlepath'] = bundlepath
    post['hugo_uniqueid'] = hashlib.md5(filepath.encode('utf-8')).hexdigest()
    return


def make_post_frontmatter(post):
    front_matter = {
        'title': post.get('title'),
        'date': post.get('post_date').isoformat(),
        'lastmod': post.get('post_date').isoformat(),
        'slug': post.get('post_name', 'UNTITLED'),
        'type': 'posts',
    }
    if post.get('excerpt'):
        front_matter['summary'] = post.get('excerpt')
    if post.get('author'):
        front_matter['author'] = post.get('author')
    if post.get('categories'):
        front_matter['categories'] = post.get('categories')
    if post.get('tags'):
        front_matter['tags'] = post.get('tags')
    if post.get('status') == 'draft':
        front_matter['draft'] = True
    post['hugo_front_matter'] = front_matter
    return


def add_resources_to_frontmatter(post: dict, allattach: dict):
    attachments = [
        p for p in allattach if p.get('post_parent') == post.get('post_id')
    ]
    if attachments:
        post['hugo_has_attachments'] = True  # redundant
        post['hugo_front_matter']['resources'] = [{
            'src':
            os.path.basename(urlparse(a.get('attachment_url')).path),
            'title':
            a.get('title')
        } for a in attachments]
        post['hugo_attachments_src'] = [
            a.get('attachment_url') for a in attachments
        ]
    return


def convert_post(post: dict):
    body = post.get('body')
    # post is HTML, so run fake wpautop on it
    body = wpautop(body)
    # Turn Wordpress shortcodes into HTML
    body = parse_shortcodes(body)
    # Parse HTML, replacing HTML attributes with Hugo shortcodes
    body, detectedhtmlimages = shortcodify(body)
    if detectedhtmlimages:
        post['hugo_has_attachments'] = True
        # add detected images to our list
        # but first, remove any that look like IMAGENAME-WWWxHHH.jpg because we probably have the original
        detectedhtmlimages = [
            a for a in detectedhtmlimages
            if not re.match(r'(.*)\-(\d+x\d+)\.(jpg|png)$', a)
        ]
        if 'hugo_attachments_src' in post:
            post['hugo_attachments_src'].extend(detectedhtmlimages)
        else:
            post['hugo_attachments_src'] = detectedhtmlimages
    # Make body into Markdown
    h = html2text.HTML2Text()
    h.images_as_html = True
    h.wrap_links = 0
    h.inline_links = 0
    body = h.handle(body).strip()
    # Un-wrap Hugo shortcodes that got line-wrapped by html2text
    body = re.sub(r'(?s)({{[\<\%].*?[\>\%]}})', lambda match: match.group(1).
                  replace('\n', ' '), body)

    parentdir, tail = os.path.split(post['hugo_filepath'])
    if not os.path.exists(parentdir):
        os.makedirs(parentdir)
    with open(post['hugo_filepath'], 'w') as f:
        f.write('---\n')
        yaml.dump(post.get('hugo_front_matter'), f)
        f.write('---\n')
        f.write(body)

    return


def download_attachments(post, blog_url):
    if post.get('hugo_bundlepath'):
        if not os.path.exists(post.get('hugo_bundlepath')):
            os.makedirs(post.get('hugo_bundlepath'))
        for u in post.get('hugo_attachments_src', []):
            fn = os.path.basename(urlparse(u).path)
            fullpath = os.path.join(post.get('hugo_bundlepath'), fn)
            # resolve relative URLs, when needed:
            u = urljoin(blog_url, u)
            r = requests.get(u, stream=True, timeout=15)
            if r.status_code == 200:
                with open(fullpath, 'wb') as f:
                    for chunk in r:
                        f.write(chunk)
                if 'last-modified' in r.headers:
                    # set file mtime to time provided by web server
                    ts = dateutil.parser.parse(
                        r.headers['last-modified']).timestamp()
                    os.utime(fullpath, (ts, ts))
            else:
                click.echo('ERROR {} on {}'.format(r.status_code, u))


def convert_comments(post):
    comments = post.get('comments')
    if comments:
        for c in comments:
            if c.get('comment_approved'):
                comment_dir = os.path.join('data', 'comments',
                                           post['hugo_uniqueid'])
                comment_fn = 'wordpress-{0:08d}.json'.format(
                    c.get('comment_id', 0))
                comment_filepath = os.path.join(comment_dir, comment_fn)
                comment_out = {}

                h = html2text.HTML2Text()
                h.wrap_links = 0

                comment_out['_id'] = hashlib.md5(
                    str(c.get('comment_id')).encode('utf-8')).hexdigest()
                if 'comment_parent' in c:
                    if c['comment_parent'] != 0:
                        comment_out['_parent'] = hashlib.md5(
                            str(c.get('comment_parent')).encode(
                                'utf-8')).hexdigest()
                    else:
                        comment_out['_parent'] = post.get('post_name')
                if post.get('post_name'):
                    comment_out['slug'] = post.get('post_name')
                comment_out['date'] = c.get(
                    'comment_date_gmt',
                    datetime.datetime(
                        1970, 1, 1, 0, 0, 0,
                        tzinfo=datetime.timezone.utc)).isoformat()
                if 'comment_author' in c:
                    comment_out['name'] = c.get('comment_author')
                if 'comment_author_email' in c:
                    comment_out['email'] = hashlib.md5(
                        c.get('comment_author_email').encode(
                            'utf-8')).hexdigest()
                if 'comment_author_url' in c:
                    comment_out['url'] = c.get('comment_author_url')
                if 'comment_content' in c:
                    # run fake wpautop on it
                    comment_body = wpautop(c['comment_content'])
                    # then convert to markdown
                    comment_out['message'] = h.handle(comment_body).strip()

                if not os.path.exists(comment_dir):
                    os.makedirs(comment_dir)
                with open(comment_filepath, 'w') as f:
                    f.write(json.dumps(comment_out, indent=2))
    return


@click.command(context_settings={'help_option_names':['-h','--help']})
@click.argument('wxr_file', type=click.Path(exists=True))
def main(wxr_file):
    """Convert a Wordpress WXR export to a Hugo site."""
    click.echo('Reading file {}...'.format(wxr_file))
    w = WXRFile(wxr_file)
    all_posts = w.get_posts()
    all_attachments = w.get_attachments()

    with click.progressbar(
            all_posts, label='Munching metadata.....', show_pos=True) as bar:
        for post in bar:
            check_post_attachments(post, all_attachments)
            make_post_destinations(post)
            make_post_frontmatter(post)
            add_resources_to_frontmatter(post, all_attachments)

    with click.progressbar(
            all_posts, label='Processing posts......', show_pos=True) as bar:
        for post in bar:
            convert_post(post)
    with click.progressbar(
            all_posts, label='Adding attachments....', show_pos=True) as bar:
        for post in bar:
            download_attachments(post, w.blog_url)
    with click.progressbar(
            all_posts, label='Converting comments...', show_pos=True) as bar:
        for post in bar:
            convert_comments(post)

    click.echo('Done.')



# Original source:
#   https://github.com/RealGeeks/wp_export_parser/blob/master/wp_export_parser/autop.py

import re

"""
This is an attempt at a 1-to-1 php-to-python translation of the wpautop function in wordpress.

I did not invent the variable names pee and tinkle.  Those are copied from the wordpress people.

"""


def clean_pre(matches):
    text = matches.group(0)
    text = re.sub('<br />', '', text)
    text = re.sub('<br/>', '', text)
    text = re.sub('<br>', '', text)
    text = re.sub('<p>', '\n', text)
    text = re.sub('</p>', '', text)

    return text


def wpautop(pee, br=1):
    if not pee or pee.strip() == '':
            return ''
    pee = pee + "\n"  # just to make things a little easier, pad the end
    pee = re.sub('<br />\s*<br />', "\n\n", pee)

    # Space things ut a little
    allblocks = '(?:table|thead|tfoot|caption|col|colgroup|tbody|tr|td|th|div|dl|dd|dt|ul|ol|li|pre|select|form|map|area|blockquote|address|math|style|input|p|h[1-6]|hr|fieldset|legend)'
    pee = re.sub('(<' + allblocks + '[^>]*>)', r"\n\1", pee)
    pee = re.sub('(</' + allblocks + '>)', r"\1\n\n", pee)
    # Cross-platform newlines
    pee = pee.replace("\r\n", "\n")
    pee = pee.replace("\r", "\n")

    if pee.find('<object') != -1:
        pee = re.sub('\s*<([^>]*)>\s*', r"<\1>", pee)  # no pee inside object/embed
        pee = re.sub('\s*</embed>\s*', '</embed>', pee)
    pee = re.sub("/\n\n+/", "\n\n", pee)  # take care of duplicates

    # make paragraphs, including one at the end
    pees = re.split('\n\s*\n', pee)
    pee = ''
    for tinkle in pees:
        pee += '<p>' + tinkle.strip("\n") + "</p>\n"
    pee = re.sub('<p>\s*</p>', '', pee)  # under certain strange conditions it could create a P of entirely whitespace
    pee = re.sub('<p>([^<]+)</(div|address|form)>', r"<p>\1</p></$2>", pee)
    pee = re.sub('<p>\s*(</?' + allblocks + '[^>]*>)\s*</p>', r"\1", pee)  # don't pee all over a tag
    pee = re.sub("<p>(<li.+?)</p>", r"\1", pee)  # problem with nested lists
    pee = re.sub('<p><blockquote([^>]*)>', r"<blockquote\1><p>", pee)
    pee = pee.replace('</blockquote></p>', '</p></blockquote>')
    pee = re.sub('<p>\s*(</?' + allblocks + '[^>]*>)', r"\1", pee)
    pee = re.sub('(</?' + allblocks + '[^>]*>)\s*</p>', r"\1", pee)
    if br:
        pee = re.sub('(?<!<br />)\s*\n', "<br />\n", pee)  # optionally make line breaks
        pee = pee.replace('<WPPreserveNewline />', "\n")
    pee = re.sub('(</?' + allblocks + '[^>]*>)\s*<br />', r"\1", pee)
    pee = re.sub('<br />(\s*</?(?:p|li|div|dl|dd|dt|th|pre|td|ul|ol)[^>]*>)', r'\1', pee)
    if pee.find('<pre') != -1:
        compiled_pre_re = re.compile('(<pre[^>]*>)(.*?)</pre>', flags=re.DOTALL)  # Note: Had to compile here since 2.6 doesn't have the DOTALL flag
        pee = re.sub(compiled_pre_re, clean_pre, pee)
    pee = re.sub("\n</p>$", '</p>', pee)

    return pee



# Original source:
#   https://github.com/RealGeeks/wp_export_parser/blob/master/wp_export_parser/parse_shortcodes.py

import re


def do_caption(tag_atts, tag_contents):
    """
    From wordpress source: https://github.com/WordPress/WordPress/blob/master/wp-includes/media.php#L620
    return '<div ' . $id . 'class="wp-caption ' . esc_attr($align) . '" style="width: ' . (10 + (int) $width) . 'px">'
    . do_shortcode( $content ) . '<p class="wp-caption-text">' . $caption . '</p></div>';
    
    """
    if not tag_atts:
        return ''
    tag_atts = dict([(a[0], a[1]) for a in tag_atts])

    if not tag_atts.get('caption'):
        # new-style caption shortcode (caption in body with image)
        match_object = re.search(
            r'((?:<a [^>]+>\s*)?<img [^>]+>(?:\s*</a>)?)(.*)',
            tag_contents,
            flags=re.IGNORECASE)
        if match_object:
            tag_contents = match_object.group(1)
            tag_atts['caption'] = match_object.group(2).strip()
    else:
        # old-style caption shortcode (caption="caption")
        #$attr['caption'] = wp_kses( $attr['caption'], 'post' );
        pass

    classes = ['wp-caption']
    if 'align' in tag_atts:
        classes.append(tag_atts.get('align'))
    return "<figure id=\"{id}\" class=\"{classes}\">{content}<figcaption class=\"wp-caption-text\">{caption}</figcaption></figure>".format(
        id=tag_atts.get('id', ''),
        classes=' '.join(classes),
        width=int(tag_atts.get('width', 0)) + 10,
        content=tag_contents,
        caption=tag_atts.get('caption', ''),
    )


TAGS_WE_CAN_PARSE = {
    'caption': do_caption,
}


def replace_tags(match):
    tag_name = match.group(2)
    tag_atts = match.group(3)
    tag_contents = match.group(5)
    if tag_name in TAGS_WE_CAN_PARSE:
        tag_atts = parse_shortcode_atts(tag_atts)
        return TAGS_WE_CAN_PARSE[tag_name](tag_atts, tag_contents)


def parse_shortcode_atts(atts):
    pattern = r'(\w+)\s*=\s*"([^"]*)"(?:\s|$)|(\w+)\s*=\s*\'([^\']*)\'(?:\s|$)|(\w+)\s*=\s*([^\s\'"]+)(?:\s|$)|"([^"]*)"(?:\s|$)|(\S+)(?:\s|$)'
    return re.findall(pattern, atts)


def parse_shortcodes(post_body):
    """
    I stole this shortcode regex from Wordpress's source.  It is very confusing.
    """
    tagregexp = '|'.join([re.escape(t) for t in TAGS_WE_CAN_PARSE.keys()])
    pattern = re.compile(
        '\\[(\\[?)(' + tagregexp +
        ')\\b([^\\]\\/]*(?:\\/(?!\\])[^\\]\\/]*)*?)(?:(\\/)\\]|\\](?:([^\\[]*(?:\\[(?!\\/\\2\\])[^\\[]*)*)\\[\\/\\2\\])?)(\\]?)'
    )
    return re.sub(pattern, replace_tags, post_body)
